####Setup####

library(dplyr)
library(magrittr)
library(ggplot2)
library(readxl)
library(tidyr)
library(estimatr)

#Note sum_outlier is a temporary variable

####Test outliers and record functions####

test_outlier <- function(model) {
  
  yhattt <- fitted(model)
  e <- residuals(model)
  lev <- hatvalues(model)
  influence <- influence.measures(model)[1] %>% as.data.frame()
  dfits <- influence$infmat.dffit
  wd <- dfits * sqrt((nobs(model)-1)/(1-lev)) #Implemented Welsch's distance following Stata manual pg. 11: https://www.stata.com/manuals/rregresspostestimation.pdf
  cooksd <- cooks.distance(model)
  
  k <- length(coef(model)) 
  
  g1 <- as.numeric(abs(dfits) > 2 * sqrt(k / nobs(model)))
  dfits_outlier_stat <- abs(dfits)
  
  g2 <- as.numeric(abs(cooksd) > 4 / nobs(model))
  cooksd_outlier_stat <- abs(cooksd)
  
  g3 <- as.numeric(abs(wd) > 3 * sqrt(k))
  wd_outlier_stat <- abs(wd)
  
  g4 <- as.numeric(lev > (2 * k) / nobs(model))
  lev_outlier_stat <- abs(lev)
  
  outlier_vars <- cbind(g1, g2, g3, g4)
  
  outlier_vars[is.na(outlier_vars)] <- 0
  
  sum_outlier <- rowSums(outlier_vars)
  
  outliers <- data.frame(
    row_number = rownames(model$model),
    sum_outlier = sum_outlier
  )
  
  outliers <- filter(outliers, sum_outlier > 1)
  
  return(outliers)
}

test_outlier_loop <- function(DataNew, Model){
  
  DataNew$N1outlier <- 0
  DataNew$sum_outlier <- 0
  
  Looptest <- 3
  
  while(Looptest >= 3){
    
    DataNew <- dplyr::select(DataNew,-sum_outlier)
    
    modelNPeak <- DataNew %>% filter(N1outlier == 0) %>% lm(Model, data = .)
    
    print(modelNPeak)
    
    DataTemp <- DataNew %>% filter(N1outlier == 0)
    
    DataTemp$row_number <- rownames(DataTemp)
    
    Outliers <- test_outlier(modelNPeak)
    
    DataTemp <- left_join(DataTemp, Outliers, by = 'row_number')
    
    DataTemp <- dplyr::select(DataTemp,edb,year,sum_outlier)
    
    DataNew <- left_join(DataNew, DataTemp, by = c('edb','year'))
    
    DataNew <- mutate(DataNew, N1outlier = ifelse(sum_outlier>= 3 | N1outlier== 1,1,0))  %>% 
      mutate(N1outlier = ifelse(is.na(N1outlier),0,N1outlier))
    
    Outliers
    
    Looptest <- max(Outliers$sum_outlier)
  }
  return(modelNPeak)
}

test_outlier_loop_robust <- function(DataNew, Model){
  
  DataNew$N1outlier <- 0
  DataNew$sum_outlier <- 0
  
  Looptest <- 3
  
  while(Looptest >= 3){
    
    DataNew <- dplyr::select(DataNew,-sum_outlier)
    
    modelNPeak <- DataNew %>% filter(N1outlier == 0) %>% lm(Model, data = .)
    
    print(modelNPeak)
    
    DataTemp <- DataNew %>% filter(N1outlier == 0)
    
    DataTemp$row_number <- rownames(DataTemp)
    
    Outliers <- test_outlier(modelNPeak)
    
    DataTemp <- left_join(DataTemp, Outliers, by = 'row_number')
    
    DataTemp <- dplyr::select(DataTemp,edb,year,sum_outlier)
    
    DataNew <- left_join(DataNew, DataTemp, by = c('edb','year'))
    
    DataNew <- mutate(DataNew, N1outlier = ifelse(sum_outlier>= 3 | N1outlier== 1,1,0))  %>% 
      mutate(N1outlier = ifelse(is.na(N1outlier),0,N1outlier))
    
    Outliers
    
    Looptest <- max(Outliers$sum_outlier)
    
    modelNRobust <- DataNew %>% filter(N1outlier == 0) %>% lm_robust(as.formula(Model), data = ., clusters = edb, se_type = "stata")
  }
  return(modelNRobust)
}

Output <- data.frame(matrix(nrow = 0, ncol = 0)) 

Record <- function(Model, Outlier){
  
  if(class(Model) == "lm"){
  
  ModelOutputs <- Model$coefficients %>% as.data.frame()
  AdjRSquared <- summary(Model)$adj.r.squared
  
  colnames(ModelOutputs) <- "Data"
  ModelOutputs$Name <- row.names(ModelOutputs)
  
  ModelOutputs <- pivot_wider(ModelOutputs, values_from = "Data", names_from = "Name")
  
  PValues <- summary(Model)$coefficients[,4] %>% as.data.frame()
  PValues$Name <- row.names(PValues)
  PValues <- pivot_wider(PValues, values_from = ".", names_from = "Name")
  colnames(PValues) <- paste0("PValue-", colnames(PValues))
  
  ModelOutputs <- bind_cols(ModelOutputs, PValues)
   
  ModelOutputs$ModelCount <- nrow(Output) + 1
  ModelOutputs$AdjRSquared  <- summary(Model)$adj.r.squared
  ModelOutputs$NetworkCost <- names(model.frame(Model))[1]
  ModelOutputs$Type <- "Not Robust"
  ModelOutputs$OutlierRemoval <- Outlier
  
  Output <<- bind_rows(Output, ModelOutputs)
  
  }
  
  if(class(Model) == "lm_robust"){
    
    ModelOutputs <- Model$coefficients %>% as.data.frame()
    AdjRSquared <- summary(Model)$adj.r.squared
    
    colnames(ModelOutputs) <- "Data"
    ModelOutputs$Name <- row.names(ModelOutputs)
    
    ModelOutputs <- pivot_wider(ModelOutputs, values_from = "Data", names_from = "Name")
    
    PValues <- Model$p.value %>% as.data.frame()
    PValues$Name <- row.names(PValues)
    PValues <- pivot_wider(PValues, values_from = ".", names_from = "Name")
    colnames(PValues) <- paste0("PValue-", colnames(PValues))
    
    ModelOutputs <- bind_cols(ModelOutputs, PValues)
    ModelOutputs$ModelCount <- nrow(Output) + 1
    ModelOutputs$AdjRSquared  <- summary(Model)$adj.r.squared
    ModelOutputs$NetworkCost <- names(model.frame(Model))[1]
    ModelOutputs$Type <- "Robust"
    ModelOutputs$OutlierRemoval <- Outlier
    
    Output <<- bind_rows(Output, ModelOutputs)
    
  }
  
  
}

####Read in data####

#Read in the new data
#Read in the inflation index

DataNew <- read.csv("Inputs/DPP4 - Opex data - v0.2.csv", header = TRUE, stringsAsFactors = FALSE)
InflationIndex <- read.csv("Inputs/InflationIndex.csv", header = TRUE, stringsAsFactors = FALSE)

####Prepare data####

#Remove impact of leases 

OpLease <- read_xlsx("Inputs/Operating-lease-model-EDB-DPP3-final-determination-27-November-2019.xlsx", sheet = "Output (opex proj)", skip = 5)
OpLease <- filter(OpLease, !is.na(Name))
colnames(OpLease)[2:3] <- c("Year","Type")
OpLease <- filter(OpLease, !is.na(Year))
OpLease <- mutate(OpLease, Name = ifelse(grepl("Non-network", Name, fixed = TRUE),"nonnetworkAdj","networkAdj"))
OpLease <- dplyr::select(OpLease, 1:2,4:20)
OpLease <- pivot_longer(OpLease,3:19)
OpLease <- OpLease %>% group_by(Name,Year,name) %>% summarise(Value = sum(value))
OpLease_nonNetwork <- filter(OpLease, Name == "nonnetworkAdj") %>% ungroup %>% dplyr::select(year = Year, edb = name, nonnetworkAdj = Value)
OpLease_Network <- filter(OpLease, Name == "networkAdj")  %>% ungroup %>% dplyr::select(year = Year, edb = name, networkAdj = Value)

DataNew <- left_join(DataNew, OpLease_nonNetwork, by = c("edb","year"))
DataNew <- left_join(DataNew, OpLease_Network, by = c("edb","year"))

DataNew[is.na(DataNew)] <- 0

DataNew <- DataNew %>% mutate(nonnetwork = nonnetwork + nonnetworkAdj, network = network + networkAdj)

#Index the data to 2019 real

DataNew <- left_join(DataNew, InflationIndex, by = 'year')

DataNew <- mutate(DataNew, network = network / Index)
DataNew <- mutate(DataNew, nonnetwork = nonnetwork / Index)

DataNew$lines <- as.numeric(DataNew$lines)

#Take logs

vars <- c("ICP", "lines", "network", "nonnetwork")
for (var in vars) {
  DataNew[[var]] <- log(DataNew[[var]])
}


####Network cost model####
#Eliminate outliers with 3+ test failures 
#Have some code here for iteration

## Model attempt 1 - No outlier elimination model

modelN <- lm(network ~ lines + ICP, data = DataNew)

ModelOutlier <- test_outlier_loop(DataNew, "network ~  lines + ICP")

Record(modelN, "No")
Record(ModelOutlier, "Yes")

#Robust SEs

lm_robust(network ~ lines + ICP, data = DataNew, clusters = edb, se_type = "stata") %>% summary()

ModelOutlierRobust <- test_outlier_loop_robust(DataNew, "network ~  lines + ICP")

Record(ModelOutlierRobust, "Yes")

####Non-Network cost model####

## Model attempt 1 - No outlier elimination model

modelNN <- lm(nonnetwork ~ lines + ICP, data = DataNew)

ModelNNOutlier <- test_outlier_loop(DataNew, "nonnetwork ~  lines + ICP")

Record(modelNN, "No")
Record(ModelNNOutlier ,"Yes")

#Robust SEs

lm_robust(nonnetwork ~ lines + ICP, data = DataNew, clusters = edb, se_type = "stata")

ModelOutlierRobust <- test_outlier_loop_robust(DataNew, "nonnetwork ~  lines + ICP")

Record(ModelOutlierRobust, "Yes")

####Alternate models####

####Peak####
#Convert peak to logs
#Network peak

DataNew$peak <- log(DataNew$peak)

modelNPeak <- lm(network ~ peak, data = DataNew)
Record(modelNPeak, "No")

modelNPeak <- lm(network ~ peak + lines + ICP, data = DataNew)
Record(modelNPeak, "No")

DataNew$row_number <- rownames(DataNew)

Outliers <- test_outlier(modelNPeak)

modelNPeak <- test_outlier_loop(DataNew, "network ~ peak + lines + ICP")
Record(modelNPeak, "Yes")

#Non-network peak

modelNPeak <- lm(nonnetwork ~ peak, data = DataNew)
Record(modelNPeak, "No")

modelNPeak <- lm(nonnetwork ~ peak + lines + ICP, data = DataNew)
Record(modelNPeak, "No")

DataNew$row_number <- rownames(DataNew)

Outliers <- test_outlier(modelNPeak)

ModelOutlier <- test_outlier_loop(DataNew, "nonnetwork ~ peak + lines + ICP")
Record(ModelOutlier, "Yes")

ModelOutlier <- test_outlier_loop_robust(DataNew, "nonnetwork ~ peak + lines + ICP")
Record(ModelOutlier, "Yes")

####Capex###

#Convert capex to real and logs
DataNew <- mutate(DataNew, ExpenditureOnAssets = ExpenditureOnAssets / Index)

DataNew$ExpenditureOnAssets <- log(DataNew$ExpenditureOnAssets)

#Network capex

modelNCapex <- lm(network ~ ExpenditureOnAssets, data = DataNew)
Record(modelNCapex, "No")

modelNCapex <- lm(network ~ ExpenditureOnAssets + lines + ICP, data = DataNew)
Record(modelNCapex, "No")

ModelOutlier <- test_outlier_loop(DataNew, "network ~ ExpenditureOnAssets + lines + ICP")
Record(ModelOutlier, "Yes")

#Non-network capex

modelNCapex <- lm(nonnetwork ~ ExpenditureOnAssets, data = DataNew)
Record(modelNCapex, "No")

modelNCapex <- lm(nonnetwork ~ ExpenditureOnAssets + lines + ICP, data = DataNew)
Record(modelNCapex, "No")

ModelOutlier <- test_outlier_loop(DataNew, "nonnetwork ~ ExpenditureOnAssets + lines + ICP")
Record(ModelOutlier, "Yes")

#Robust SEs

lm_robust(nonnetwork ~ ExpenditureOnAssets + lines + ICP, data = DataNew, clusters = edb, se_type = "stata")

ModelOutlier <- test_outlier_loop_robust(DataNew, "nonnetwork ~ ExpenditureOnAssets + lines + ICP")
Record(ModelOutlier, "Yes")

####Delivery###
#Convert delivery to logs

DataNew$delivery <- log(DataNew$delivery)

modelNDelivery <- lm(network ~ delivery, data = DataNew)
Record(modelNDelivery, "No")
modelNDelivery <- lm(network ~ delivery + lines + ICP, data = DataNew)
Record(modelNDelivery, "No")
ModelOutlier <- test_outlier_loop(DataNew, "network ~ delivery + lines + ICP")
Record(ModelOutlier, "Yes")

modelNNDelivery <- lm(nonnetwork ~ delivery, data = DataNew)
Record(modelNNDelivery, "No")
modelNNDelivery <- lm(nonnetwork ~ delivery + lines + ICP, data = DataNew)
Record(modelNNDelivery, "No")
ModelOutlier <- test_outlier_loop(DataNew, "nonnetwork ~ delivery + lines + ICP")
Record(ModelOutlier, "Yes")

####Cost breakdown####

#no.arr = asset replacement and renewal. (Network)
#no.rcm = Routine corrective maintenace and inspection. (Network)
#n.sie = Service interruptions and maintance (Network)
#no.vm = Vegetation management (Network)
#nno.bs = Business support (non-network)
#nno.sons = System operations and support (non-network)

#Convert all to real and logs

DataNew <- mutate(DataNew, no.arr = no.arr / Index)
DataNew <- mutate(DataNew, no.rcm = no.rcm / Index)
DataNew <- mutate(DataNew, n.sie = n.sie / Index)
DataNew <- mutate(DataNew, no.vm = no.vm / Index)
DataNew <- mutate(DataNew, nno.bs = nno.bs / Index)
DataNew <- mutate(DataNew, nno.sons = nno.sons / Index)

DataNew$no.arr <- log(DataNew$no.arr)
DataNew$no.rcm <- log(DataNew$no.rcm)
DataNew$n.sie <- log(DataNew$n.sie)
DataNew$no.vm <- log(DataNew$no.vm)
DataNew$nno.bs <- log(DataNew$nno.bs)
DataNew$nno.sons <- log(DataNew$nno.sons)

#Remove infinites for each model estimate

modelCost <- DataNew %>% filter(!is.infinite(no.arr)) %>% lm(no.arr ~ lines + ICP, data = .)
Record(modelCost, "No")
modelCost <- DataNew %>% filter(!is.infinite(no.arr)) %>% test_outlier_loop(., "no.arr ~ lines + ICP")
Record(modelCost, "Yes")

modelCost <- DataNew %>% filter(!is.infinite(no.rcm)) %>% lm(no.rcm ~ lines + ICP, data = .)
Record(modelCost, "No")
modelCost <- DataNew %>% filter(!is.infinite(no.rcm)) %>% test_outlier_loop(., "no.rcm ~ lines + ICP")
Record(modelCost, "Yes")

modelCost <- DataNew %>% filter(!is.infinite(n.sie)) %>% lm(n.sie ~ lines + ICP, data = .)
Record(modelCost, "No")
modelCost <- DataNew %>% filter(!is.infinite(n.sie)) %>% test_outlier_loop(., "n.sie ~ lines + ICP")
Record(modelCost, "Yes")

modelCost <- DataNew %>% filter(!is.infinite(no.vm)) %>% lm(no.vm ~ lines + ICP, data = .)
Record(modelCost, "No")
modelCost <- DataNew %>% filter(!is.infinite(no.vm)) %>% test_outlier_loop(., "no.vm ~ lines + ICP")
Record(modelCost, "Yes")

modelCost <- DataNew %>% filter(!is.infinite(nno.bs)) %>% lm(nno.bs ~ lines + ICP, data = .)
Record(modelCost, "No")
modelCost <- DataNew %>% filter(!is.infinite(nno.bs)) %>% test_outlier_loop(., "nno.bs ~ lines + ICP")
Record(modelCost, "Yes")

modelCost <- DataNew %>% filter(!is.infinite(nno.sons)) %>% lm(nno.sons ~ lines + ICP, data = .)
Record(modelCost, "No")
modelCost <- DataNew %>% filter(!is.infinite(nno.sons)) %>% test_outlier_loop(., "nno.sons ~ lines + ICP")
Record(modelCost, "Yes")

#Capex models for cost breakdown models

modelCost <- DataNew %>% filter(!is.infinite(nno.bs)) %>% lm(nno.bs ~ ExpenditureOnAssets + lines + ICP, data = .)
Record(modelCost, "No")
modelCost <- DataNew %>% filter(!is.infinite(nno.bs)) %>% test_outlier_loop(., "nno.bs ~ ExpenditureOnAssets + lines + ICP")
Record(modelCost, "Yes")
modelCost <- DataNew %>% filter(!is.infinite(nno.bs)) %>% test_outlier_loop(., "nno.bs ~ ExpenditureOnAssets + ICP")
Record(modelCost, "Yes")

modelCost <- DataNew %>% filter(!is.infinite(nno.sons)) %>% lm(nno.sons ~ ExpenditureOnAssets + lines + ICP, data = .)
Record(modelCost, "No")
modelCost <- DataNew %>% filter(!is.infinite(nno.sons)) %>% test_outlier_loop(., "nno.sons ~ ExpenditureOnAssets + lines + ICP")
Record(modelCost, "Yes")
modelCost <- DataNew %>% filter(!is.infinite(nno.bs)) %>% test_outlier_loop(., "nno.sons ~ ExpenditureOnAssets + ICP")
Record(modelCost, "Yes")

####Outputs####

#Re-order columns

Output <- select(Output, ModelCount, everything())

#Replace NAs with ""

Output[is.na(Output)] <- ""

#Write output

write.csv(Output, "Outputs/Output.csv", row.names = FALSE)
